# Tcl autoload index file, version 2.0
# generated from tclIndex by
# regsub {\[list source \[file join \$dir ([^\.\r]+)\.tcl\]\]} {[list source -rsrc \1]}


set auto_index(::vfs::ftp::Mount) [list source -rsrc ftpvfs]
set auto_index(::vfs::ftp::Unmount) [list source -rsrc ftpvfs]
set auto_index(::vfs::ftp::handler) [list source -rsrc ftpvfs]
set auto_index(::vfs::ftp::stat) [list source -rsrc ftpvfs]
set auto_index(::vfs::ftp::access) [list source -rsrc ftpvfs]
set auto_index(::vfs::ftp::open) [list source -rsrc ftpvfs]
set auto_index(::vfs::ftp::_closing) [list source -rsrc ftpvfs]
set auto_index(::vfs::ftp::_findFtpInfo) [list source -rsrc ftpvfs]
set auto_index(::vfs::ftp::matchindirectory) [list source -rsrc ftpvfs]
set auto_index(::vfs::ftp::createdirectory) [list source -rsrc ftpvfs]
set auto_index(::vfs::ftp::removedirectory) [list source -rsrc ftpvfs]
set auto_index(::vfs::ftp::deletefile) [list source -rsrc ftpvfs]
set auto_index(::vfs::ftp::fileattributes) [list source -rsrc ftpvfs]
set auto_index(::vfs::ftp::utime) [list source -rsrc ftpvfs]
set auto_index(::vfs::http::Mount) [list source -rsrc httpvfs]
set auto_index(::vfs::http::Unmount) [list source -rsrc httpvfs]
set auto_index(::vfs::http::handler) [list source -rsrc httpvfs]
set auto_index(::vfs::http::stat) [list source -rsrc httpvfs]
set auto_index(::vfs::http::access) [list source -rsrc httpvfs]
set auto_index(::vfs::http::open) [list source -rsrc httpvfs]
set auto_index(::vfs::http::_findHttpInfo) [list source -rsrc httpvfs]
set auto_index(::vfs::http::matchindirectory) [list source -rsrc httpvfs]
set auto_index(::vfs::http::createdirectory) [list source -rsrc httpvfs]
set auto_index(::vfs::http::removedirectory) [list source -rsrc httpvfs]
set auto_index(::vfs::http::deletefile) [list source -rsrc httpvfs]
set auto_index(::vfs::http::fileattributes) [list source -rsrc httpvfs]
set auto_index(::vfs::http::utime) [list source -rsrc httpvfs]
set auto_index(::vfs::mk4::Mount) [list source -rsrc mk4vfs]
set auto_index(::vfs::mk4::Unmount) [list source -rsrc mk4vfs]
set auto_index(::vfs::mk4::handler) [list source -rsrc mk4vfs]
set auto_index(::vfs::mk4::utime) [list source -rsrc mk4vfs]
set auto_index(::vfs::mk4::matchindirectory) [list source -rsrc mk4vfs]
set auto_index(::vfs::mk4::stat) [list source -rsrc mk4vfs]
set auto_index(::vfs::mk4::access) [list source -rsrc mk4vfs]
set auto_index(::vfs::mk4::open) [list source -rsrc mk4vfs]
set auto_index(::vfs::mk4::createdirectory) [list source -rsrc mk4vfs]
set auto_index(::vfs::mk4::removedirectory) [list source -rsrc mk4vfs]
set auto_index(::vfs::mk4::deletefile) [list source -rsrc mk4vfs]
set auto_index(::vfs::mk4::fileattributes) [list source -rsrc mk4vfs]
set auto_index(::mk4vfs::init) [list source -rsrc mk4vfs]
set auto_index(::mk4vfs::mount) [list source -rsrc mk4vfs]
set auto_index(::mk4vfs::_commit) [list source -rsrc mk4vfs]
set auto_index(::mk4vfs::umount) [list source -rsrc mk4vfs]
set auto_index(::mk4vfs::stat) [list source -rsrc mk4vfs]
set auto_index(::mk4vfs::driver) [list source -rsrc mk4vfs]
set auto_index(::mk4vfs::do_close) [list source -rsrc mk4vfs]
set auto_index(::mk4vfs::mkdir) [list source -rsrc mk4vfs]
set auto_index(::mk4vfs::getdir) [list source -rsrc mk4vfs]
set auto_index(::mk4vfs::mtime) [list source -rsrc mk4vfs]
set auto_index(::mk4vfs::delete) [list source -rsrc mk4vfs]
set auto_index(::mk4vfs::find/file) [list source -rsrc mk4vfs]
set auto_index(::mk4vfs::find/dir) [list source -rsrc mk4vfs]
set auto_index(::scripdoc::init) [list source -rsrc scripdoc]
set auto_index(::scripdoc::extendPath) [list source -rsrc scripdoc]
set auto_index(::vfs::ns::Mount) [list source -rsrc tclprocvfs]
set auto_index(::vfs::ns::Unmount) [list source -rsrc tclprocvfs]
set auto_index(::vfs::ns::handler) [list source -rsrc tclprocvfs]
set auto_index(::vfs::ns::stat) [list source -rsrc tclprocvfs]
set auto_index(::vfs::ns::access) [list source -rsrc tclprocvfs]
set auto_index(::vfs::ns::exists) [list source -rsrc tclprocvfs]
set auto_index(::vfs::ns::open) [list source -rsrc tclprocvfs]
set auto_index(::vfs::ns::_generate) [list source -rsrc tclprocvfs]
set auto_index(::vfs::ns::matchindirectory) [list source -rsrc tclprocvfs]
set auto_index(::vfs::ns::createdirectory) [list source -rsrc tclprocvfs]
set auto_index(::vfs::ns::removedirectory) [list source -rsrc tclprocvfs]
set auto_index(::vfs::ns::deletefile) [list source -rsrc tclprocvfs]
set auto_index(::vfs::ns::fileattributes) [list source -rsrc tclprocvfs]
set auto_index(::vfs::ns::utime) [list source -rsrc tclprocvfs]
set auto_index(::vfs::test::Mount) [list source -rsrc testvfs]
set auto_index(::vfs::test::Unmount) [list source -rsrc testvfs]
set auto_index(::vfs::test::handler) [list source -rsrc testvfs]
set auto_index(::vfs::test::stat) [list source -rsrc testvfs]
set auto_index(::vfs::test::access) [list source -rsrc testvfs]
set auto_index(::vfs::test::open) [list source -rsrc testvfs]
set auto_index(::vfs::test::matchindirectory) [list source -rsrc testvfs]
set auto_index(::vfs::test::createdirectory) [list source -rsrc testvfs]
set auto_index(::vfs::test::removedirectory) [list source -rsrc testvfs]
set auto_index(::vfs::test::deletefile) [list source -rsrc testvfs]
set auto_index(::vfs::test::fileattributes) [list source -rsrc testvfs]
set auto_index(::vfs::test::utime) [list source -rsrc testvfs]
set auto_index(::vfs::debug) [list source -rsrc vfs]
set auto_index(::vfs::urltype::Mount) [list source -rsrc vfsUrl]
set auto_index(::vfs::urltype::handler) [list source -rsrc vfsUrl]
set auto_index(::vfs::urltype::stat) [list source -rsrc vfsUrl]
set auto_index(::vfs::urltype::open) [list source -rsrc vfsUrl]
set auto_index(::vfs::urltype::access) [list source -rsrc vfsUrl]
set auto_index(::vfs::urltype::matchindirectory) [list source -rsrc vfsUrl]
set auto_index(::vfs::urltype::createdirectory) [list source -rsrc vfsUrl]
set auto_index(::vfs::urltype::removedirectory) [list source -rsrc vfsUrl]
set auto_index(::vfs::urltype::deletefile) [list source -rsrc vfsUrl]
set auto_index(::vfs::urltype::fileattributes) [list source -rsrc vfsUrl]
set auto_index(::vfs::urltype::utime) [list source -rsrc vfsUrl]
set auto_index(::vfs::listVolumes) [list source -rsrc vfsUtils]
set auto_index(::vfs::addVolume) [list source -rsrc vfsUtils]
set auto_index(::vfs::removeVolume) [list source -rsrc vfsUtils]
set auto_index(::vfs::autoMountExtension) [list source -rsrc vfsUtils]
set auto_index(::vfs::autoMountUrl) [list source -rsrc vfsUtils]
set auto_index(::vfs::log) [list source -rsrc vfsUtils]
set auto_index(::vfs::RegisterMount) [list source -rsrc vfsUtils]
set auto_index(::vfs::unmount) [list source -rsrc vfsUtils]
set auto_index(::vfs::haveMount) [list source -rsrc vfsUtils]
set auto_index(::vfs::urlMount) [list source -rsrc vfsUtils]
set auto_index(::vfs::fileUrlMount) [list source -rsrc vfsUtils]
set auto_index(::vfs::tclprocMount) [list source -rsrc vfsUtils]
set auto_index(::vfs::auto) [list source -rsrc vfsUtils]
set auto_index(::vfs::matchCorrectTypes) [list source -rsrc vfsUtils]
set auto_index(::vfs::accessMode) [list source -rsrc vfsUtils]
set auto_index(::vfs::matchDirectories) [list source -rsrc vfsUtils]
set auto_index(::vfs::matchFiles) [list source -rsrc vfsUtils]
set auto_index(::vfs::modeToString) [list source -rsrc vfsUtils]
set auto_index(::vfs::listAttributes) [list source -rsrc vfsUtils]
set auto_index(::vfs::indexToAttribute) [list source -rsrc vfsUtils]
set auto_index(::vfs::attributesGet) [list source -rsrc vfsUtils]
set auto_index(::vfs::attributesSet) [list source -rsrc vfsUtils]
set auto_index(::vfs::posixError) [list source -rsrc vfsUtils]
set auto_index(::vfs::zip::Execute) [list source -rsrc zipvfs]
set auto_index(::vfs::zip::Mount) [list source -rsrc zipvfs]
set auto_index(::vfs::zip::Unmount) [list source -rsrc zipvfs]
set auto_index(::vfs::zip::handler) [list source -rsrc zipvfs]
set auto_index(::vfs::zip::matchindirectory) [list source -rsrc zipvfs]
set auto_index(::vfs::zip::stat) [list source -rsrc zipvfs]
set auto_index(::vfs::zip::access) [list source -rsrc zipvfs]
set auto_index(::vfs::zip::open) [list source -rsrc zipvfs]
set auto_index(::vfs::zip::createdirectory) [list source -rsrc zipvfs]
set auto_index(::vfs::zip::removedirectory) [list source -rsrc zipvfs]
set auto_index(::vfs::zip::deletefile) [list source -rsrc zipvfs]
set auto_index(::vfs::zip::fileattributes) [list source -rsrc zipvfs]
set auto_index(::vfs::zip::utime) [list source -rsrc zipvfs]
set auto_index(::zip::u_short) [list source -rsrc zipvfs]
set auto_index(::zip::DosTime) [list source -rsrc zipvfs]
set auto_index(::zip::Data) [list source -rsrc zipvfs]
set auto_index(::zip::EndOfArchive) [list source -rsrc zipvfs]
set auto_index(::zip::TOC) [list source -rsrc zipvfs]
set auto_index(::zip::open) [list source -rsrc zipvfs]
set auto_index(::zip::FAKEDIR) [list source -rsrc zipvfs]
set auto_index(::zip::exists) [list source -rsrc zipvfs]
set auto_index(::zip::stat) [list source -rsrc zipvfs]
set auto_index(::zip::getdir) [list source -rsrc zipvfs]
set auto_index(::zip::_close) [list source -rsrc zipvfs]
